//
// idt_r
//
// Treiber zum Auslesen der Interrupt Descriptor Table
//
// written 1998 by Guido Wischrop
//

#include "ntddk.h"
#include "idt_r.h"

// Macro fuer die Debug-Ausgabe - Ausgabe erfolgt nur im "Checked-Build"
#if DBG
#define DbgPrint(arg) DbgPrint arg
#else
#define DbgPrint(arg)
#endif


NTSTATUS IDTRead_DispatchCreateClose(IN PDEVICE_OBJECT devObj,IN PIRP Irp)

{
    DbgPrint(("IDTRead: Open / Close\n")); 
    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = STATUS_SUCCESS;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return STATUS_SUCCESS;
}



NTSTATUS IDTRead_DispatchDeviceControl(IN PDEVICE_OBJECT devObj,IN PIRP Irp)

{


    UCHAR *mBuffer,*mVer=mVersion;
    ULONG mInputBufferLength, mOutputBufferLength, dummy, mLoopCount;
    ULONG mIOCTLCode;
    NTSTATUS sts;
    
    PIO_STACK_LOCATION IrpStack = IoGetCurrentIrpStackLocation(Irp);
    
    mIOCTLCode=IrpStack->Parameters.DeviceIoControl.IoControlCode;
    mInputBufferLength=IrpStack->Parameters.DeviceIoControl.InputBufferLength;
    mOutputBufferLength=IrpStack->Parameters.DeviceIoControl.OutputBufferLength;

    DbgPrint(("IDTRead: InputBufferLength = %x\n",mInputBufferLength));
    DbgPrint(("IDTRead: OutputBufferLength = %x\n",mOutputBufferLength));
    DbgPrint(("IDTRead: FunctionCode = %x\n",mIOCTLCode));
    

    if (mIOCTLCode==IOCTL_IDTRead_GetIDT)
      {

        mBuffer=Irp->AssociatedIrp.SystemBuffer;
        _asm
          {
                mov   esi,mBuffer
                sidt  [esi]
                movzx ecx,word ptr [esi]
                mov   ebx,dword ptr [esi+2]
                inc   ecx                
                mov   edi,mBuffer
                mov   dummy,ecx
                mov   esi,ebx
                shr   ecx,2  
                rep   movsd
          }
          
        Irp->IoStatus.Information = dummy;
        DbgPrint(("IDTRead: Laenge %x - 1. Bytes gelesen: %x\n",
                  dummy,*mBuffer));
      } else Irp->IoStatus.Information = 0;


    Irp->IoStatus.Status = STATUS_SUCCESS;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return STATUS_SUCCESS;
}


VOID IDTRead_Unload( IN PDRIVER_OBJECT DrvObj)
{
     WCHAR LinkName[]=L"\\DosDevices\\IDTRead";
     UNICODE_STRING uniLinkName;
     NTSTATUS sts;     

     DbgPrint(("IDTRead: Eintritt in Unload Routine !\n"));

     RtlInitUnicodeString(&uniLinkName, LinkName);
 
     sts=IoDeleteSymbolicLink(&uniLinkName);
     if (!NT_SUCCESS(sts))
      {
        DbgPrint(("IDTRead: Kann Link nicht entfernen !\n"));
        return;
      } else DbgPrint(("IDTRead: Link entfernt !\n"));
     
     IoDeleteDevice(DrvObj->DeviceObject);
     
     DbgPrint(("IDTRead: Unload Ende !\n"));

}


NTSTATUS DriverEntry( IN PDRIVER_OBJECT drvObj,
                      IN PUNICODE_STRING RegistryPath)

{
     PDEVICE_OBJECT devObj;
     NTSTATUS sts;
     WCHAR Name[]=L"\\Device\\IDTRead";
     WCHAR DOSName[]=L"\\DosDevices\\IDTRead";
     UNICODE_STRING uniName, uniDOSName;

     RtlInitUnicodeString(&uniName, Name);
     RtlInitUnicodeString(&uniDOSName, DOSName);
     
     drvObj->MajorFunction[IRP_MJ_CREATE]=IDTRead_DispatchCreateClose;
     drvObj->MajorFunction[IRP_MJ_CLOSE]=IDTRead_DispatchCreateClose;
     drvObj->MajorFunction[IRP_MJ_DEVICE_CONTROL]=IDTRead_DispatchDeviceControl;
     
     drvObj->DriverUnload=IDTRead_Unload;
     
     sts=IoCreateDevice(drvObj,
                        0,
                        &uniName,
                        FILE_DEVICE_UNKNOWN,
                        0,
                        TRUE,
                        &devObj);
     if (!NT_SUCCESS(sts))
      {
        DbgPrint(("IDTRead: Kann DeviceObject nicht erstellen !\n"));
        return sts;
      } 

     sts=IoCreateSymbolicLink(&uniDOSName, &uniName);
     if (!NT_SUCCESS(sts))
      {
        DbgPrint(("IDTRead: Kann Link nicht erstellen !\n"));
        return sts;
      } else
      {
        DbgPrint(("IDTRead: Link: %s - DosLink: %s\n", Name, DOSName));
      } 

    devObj->Flags |= DO_BUFFERED_IO;
    
    DbgPrint(("IDTRead: Driver Entry erfolgreich abgearbeitet !\n"));

    return STATUS_SUCCESS;
}     

